'
' -----[ Program Description ]--------------------------------------------------
'
' This program implements a a mutiplexer box for 3x RS232 and 2x RS485.
' Used for ModBus RTU protocol and BASCOM universal I/O devices
' MCU board is a simmstick DT104 with ISP and simpliest part list.
' It controlls AVR_RS232_Mux.sch / AVR_RS232_Mux.brd  for slave testing purpose.
' It's based on the idea of an PC as busmaster, and max tree RS232 slaves and
' two lines of RS485 slaves. With choosen quartz 38400 baud run sometimes only
' a bit eroneus. For the working speed of 9600 it's error free.
'
' -----[ Disclaimer ]-----------------------------------------------------------
'
' This example is offered on an "AS IS" basis, no warranty expressed or implied.
' The programers disclaim liability of any damages associated with the use of
' the hardware or software described herein.
' You use it on your own risk. We are not able to provide any free support.
'
' Copyright (c) 2002 Mike Eitel all rights reserved
'
' -----[ Revision History ]-----------------------------------------------------

' 020621 - Version 1.00AVR            Ver 1.00 RS232-RS485 SerialMux  Mike Eitel

' -----[ Aliases ]--------------------------------------------------------------

' Define basics and serial communication parameters
  $regfile = "2313def.dat"                   'regfile must match the chip
  $crystal = 14318180                        'crystal must match too

  ' Distribution of port pins is forced by single layer pcb
  ' 8 LED's show the activities of the communication channels
  Out_rs485_dev_4 Alias Portb.0              ' PB0 Out Dev 4 TXD
  Snd_rs485_dev_4 Alias Portb.1              ' PB1 Out Dev 4 send
  In_rs485_dev_4 Alias Pinb.2                ' PB2 In  Dev 4 RXD       LED 5
  Out_rs232_dev_3 Alias Portb.3              ' PB3 Out Dev 3 TXD
  Out_rs232_dev_2 Alias Portb.4              ' PB4 Out Dev 2 TXD
  Out_rs232_dev_1 Alias Portb.5              ' PB5 Out Dev 1 TXD
  In_rs232_dev_3 Alias Pinb.6                ' PB6 In  Dev 3 RXD       LED 4
  In_rs232_dev_2 Alias Pinb.7                ' PB7 In  Dev 2 RXD       LED 3

  In_rs232_pc Alias Pind.0                   ' PD0 In  Main RXD        LED 8
  Out_rs232_pc Alias Portd.1                 ' PD1 Out Main TXD        LED 7
  In_rs232_dev_1 Alias Pind.2                ' PD2 In  Dev 1 RXD       LED 2
  Rts_rs232_pc Alias Pind.3                  ' PD3 In  Main RTS Signal LED 1
  In_rs485_dev_5 Alias Pind.4                ' PD4 In  Dev 5 RXD       LED 6
  Out_rs485_dev_5 Alias Portd.5              ' PD5 Out Dev 5 TXD
  Snd_rs485_dev_5 Alias Portd.6              ' PD6 Out Dev 5 send

                                             ' LED 1  PD3  Main  RTS
                                             ' LED 2  PD2  Dev 1 RXD
                                             ' LED 3  PB7  Dev 2 RXD
                                             ' LED 4  PB6  Dev 3 RXD
                                             ' LED 5  PB2  Dev 4 RXD
                                             ' LED 6  PD4  Dev 5 RXD
                                             ' LED 7  PD1  Main  TXD
                                             ' LED 8  PD0  Main  RXD

' -----[ Constants ]------------------------------------------------------------
' none
' -----[ Variables ]------------------------------------------------------------

  Dim Out_main As Bit                        ' Main TXD tmp
  Dim H_out_main As Bit                      ' Main TXD tmp 2
  Dim Sim_in_rts As Bit                      ' Main Simulate RTS read
  Dim Sim_out_rts As Bit                     ' Main Simulate RTS send
  Dim Br_main As Word                        ' Main Break TXD prolongation time

' -----[ Initialization ]-------------------------------------------------------

Begin:

' Define port parameters
' Port B is used as distributed
  Ddrb = &B00111011                          ' Use pin 0,1,3,4,5 as outputs

' Port D is used for ISP , normal RS232 pins and as distributed
  Ddrd = &B01100010                          ' Use pin 1,5,6 outputs

' Setup interrupts to defined startup
  Disable Interrupts

  '------[ Program ]------------------------------------------------------------



'  *****************************************************************************
' NOT USABLE START OF PROGRAM
' As soon as a print command us used or $baud is inserted the looping program
' does not run anymore. Printing seems to block the pins
'
'  Dim In_main As Bit                         ' In Main  RXD pin
'  Dim In_rts As Bit                          ' In Main  RTS pin
'  Dim In_dev_1 As Bit                        ' In Dev 1 RXD high value
'  Dim In_dev_2 As Bit                        ' In Dev 2 RXD high value
'  Dim In_dev_3 As Bit                        ' In Dev 3 RXD high value
'  Dim In_dev_4 As Bit                        ' In Dev 4 RXD high value
'  Dim In_dev_5 As Bit                        ' In Dev 5 RXD high value
'  In_rts = Rts_rs232_pc
'  In_main = In_rs232_pc
'  In_dev_1 = In_rs232_dev_1
'  In_dev_2 = In_rs232_dev_2
'  In_dev_3 = In_rs232_dev_3
'  In_dev_4 = In_rs485_dev_4
'  In_dev_5 = In_rs485_dev_5

'  Wait 2
'  print
'  print "Test der Inputs"
'  print "RTS Main = " ; Rts_rs232_pc
'  print "In Main  = " ; In_main
'  print "In Dev 1 = " ; In_dev_1
'  print "In Dev 2 = " ; In_dev_2
'  print "In Dev 3 = " ; In_dev_3
'  print "In Dev 4 = " ; In_dev_4
'  print "In Dev 5 = " ; In_dev_1
'  Wait 2
'  *****************************************************************************


Endless:
    '             1 = no bit active, !! reverse logic in RS232
   ' Copy PC input to outputs and set a prolongation timer to guaranty a
   ' minimum send cycle on RS485 bus
   If In_rs232_pc = 1 Then                   ' No bit to send
      Set Out_rs232_dev_1                    ' Copy to channel 1
      Set Out_rs232_dev_2                    ' Copy to channel 2
      Set Out_rs232_dev_3                    ' Copy to channel 3
      Set Out_rs485_dev_4                    ' Copy to channel 4
      Set Out_rs485_dev_5                    ' Copy to channel 5
      If Br_main <= 2 Then                   ' Revers RS485 channel direction
         Br_main = 1                         ' RTS timer inactive
         Sim_out_rts = 0                     ' Set channel to read mode
      End If
      Decr Br_main                           ' Reduce send-timer
   Else                                      ' Aktive pegel recognized
      Reset Out_rs232_dev_1                  ' Copy to channel 1
      Reset Out_rs232_dev_2                  ' Copy to channel 2
      Reset Out_rs232_dev_3                  ' Copy to channel 3
      Reset Out_rs485_dev_4                  ' Copy to channel 4
      Reset Out_rs485_dev_5                  ' Copy to channel 5
      Br_main = 50                           ' Set send-active timer
      Sim_out_rts = 1                        ' Set channel to send mode
   End If

   ' Handle direction pins of the RS485 chips
   Sim_out_rts = Sim_out_rts Or Rts_rs232_pc ' Simulated or requested RTS
   Sim_in_rts = Not Sim_out_rts              ' Make inverted helper

   Snd_rs485_dev_4 = Sim_out_rts             ' Means sending channel 4
   Snd_rs485_dev_5 = Sim_out_rts             ' Means sending channel 5

   ' Combine input pins of the RS485 chips with receive direction
   ' and combine rest of RS232 pins to the common PC TXD output
   H_out_main = In_rs485_dev_4 Or Sim_out_rts       ' Use only when 4 reads
   Out_main = In_rs485_dev_5 Or Sim_out_rts  ' Read when channel 5 in read mode
   Out_main = Out_main And H_out_main        ' Combine both RS485
   Out_main = Out_main And In_rs232_dev_1    ' Combine with RS232 1
   Out_main = Out_main And In_rs232_dev_2    ' Combine with RS232 2
   Out_main = Out_main And In_rs232_dev_3    ' Combine with RS232 3
   Out_rs232_pc = Out_main                   ' Necessary cause of BASCOM parsing
                                             ' incompatible variable typ ????
   Goto Endless                              ' Ethernal Loop :-)

End